"""Main module."""

import locale
import sys
from imdb import IMDb
from collections import OrderedDict
import mkvimdb.tagify as tagify
import mkvimdb.imdbops as imdbops


def ERR(outstring):
        sys.stderr.write(str(outstring) + '\n')


class moviecursor:
    def __init__(
            self, title='Pulp Fiction',
            year='1994',
            output_dir='',
            extragenre='',
            replacegenre='',
            replacetitle='',
            interactive=False,
            verbose=False):
        self.title = title
        self.year = year
        self.output_dir = output_dir
        self.extragenre = extragenre
        self.replacegenre = replacegenre
        self.replacetitle = replacetitle
        self.interactive = interactive
        self.verbose = verbose
        self.data = {}
        self.tagdata = OrderedDict()
        self.tagdata['TITLE'] = ''
        self.tagdata['ARTIST'] = ''
        self.tagdata['DIRECTOR'] = ''
        self.tagdata['ALBUM_ARTIST'] = ''
        self.tagdata['PLOT'] = ''
        self.tagdata['COMMENTARY'] = ''
        self.tagdata['DESCRIPTION'] = ''
        self.tagdata['DATE_RELEASED'] = ''
        self.tagdata['GENRE'] = ''
        self.searchstring = "%s (%s)" % (self.title, self.year)
        self.makecursor()
        self.matchingnames()
        self.pickone()
        self.genout()
        self.overrides()
        self.spit()
    def makecursor(self):
        self.c = IMDb()
    def matchingnames(self):
        self.matches = imdbops.matchingnames(self.c, self.searchstring)
    def pickone(self):
        self.entry = imdbops.pickone(
            self.matches,
            self.title,
            self.year,
            self.interactive,
            self.verbose)
        self.entry = self.c.get_movie(self.entry.getID())
    def genout(self):
        self.tagdata.update(imdbops.genout(self.entry))
    def overrides(self):
        if self.replacegenre:
            self.tagdata['GENRE'] = self.replacegenre
        if self.extragenre:
            self.tagdata['GENRE'] = self.extragenre + ', ' + self.tagdata['GENRE']
        if self.replacetitle:
            self.tagdata['TITLE'] = self.replacetitle
    def spit(self):
        if self.output_dir:
            with open(self.tagdata['filename'] + '.xml', 'w') as FH:
                FH.write(tagify.givexml(self.tagdata))
        else:
            print(tagify.givexml(self.tagdata))





"""Tagify module."""

import xml.etree.ElementTree as ET
import xml.dom.minidom


def givexml(indict={}):
    """
    Receive a dict-ish object and spit XML
    (removed lxml dependency despite my love for lxml)
    """
    top = ET.Element('Tags')

    comment = ET.Comment('Generated by mkvimdb')
    top.append(comment)

    tag = ET.SubElement(top, 'Tag')
    target = ET.SubElement(tag, 'Targets')
    targettype = ET.SubElement(target, 'TargetTypeValue')
    targettype.text = '50'
    for movietag in indict.keys():
        simple = ET.SubElement(tag, 'Simple')
        simplename = ET.SubElement(simple, 'Name')
        simplename.text = str(movietag)
        simplestring = ET.SubElement(simple, 'String')
        simplestring.text = str(indict[movietag])

    outxml = xml.dom.minidom.parseString(ET.tostring(top))
    return(outxml.toprettyxml())



"""IMDB operations module."""

import locale
import sys
import string
from imdb import IMDb


def ERR(outstring):
        sys.stderr.write(str(outstring) + '\n')


def genout(filmentry):
    outdict={}
    kind = filmentry.get('kind')
    try:
        director = ', '.join(map(lambda x: x['name'], filmentry['directors']))
    except:
        try:
            director = ', '.join(map(lambda x: x['name'], filmentry['director']))
        except:
            director = ''
    outdict['DIRECTOR'] = director
    outdict['ALBUM_ARTIST'] = director
    try:
        directorscanon = '-'.join(map(lambda x: x['canonical name'].replace(" ","").lower(), filmentry['directors']))
    except:
        try:
            directorscanon = '-'.join(map(lambda x: x['canonical name'].replace(" ","").lower(), filmentry['director']))
        except:
            directorscanon = 'unknown'
    outdict['directorscanon'] = directorscanon
    outdict['ARTIST'] = ', '.join(map(lambda x: x['name'], filmentry['actors'][0:3]))
    try:
        plot = filmentry['plot outline']
    except:
        try:
            plot = filmentry['plot']
        except:
            try:
                plot = filmentry['synopsis']
            except:
                plot = ''
    outdict['PLOT'] = plot
    outdict['COMMENTARY'] = plot
    outdict['DESCRIPTION'] = plot
    outdict['GENRE'] = ', '.join(filmentry['genre'])
    year = str(filmentry['year'])
    outdict['DATE_RELEASED'] = year
    outdict['TITLE'] = "%s %s" % (filmentry['canonical title'], year)
    titlecanon = filmentry['smart canonical title'].lower().replace(" ","_")
    outdict['titlecanon'] = titlecanon
    if kind == "episode":
        try:
            digits = len(str(filmentry.get('number of episodes')))
        except:
            digits = 2
        outdict['SERIES'] = filmentry.get('smart canonical series title')
        outdict['seriescanon'] = outdict['SERIES'].lower().replace(" ","_")
        outdict['season'] = filmentry.get('season')
        outdict['episode'] = filmentry.get('episode')
        filetemplate = "%s-S%02dE%0" + str(digits) + "d-%s"
        titletemplate = "S%02dE%0" + str(digits) + "d %s"
        outdict['TITLE'] = titletemplate % (int(outdict['season']), int(outdict['episode']), outdict['TITLE'])
        outdict['filename'] = filetemplate % (
            outdict['seriescanon'],
            int(outdict['season']),
            int(outdict['episode']),
            outdict['titlecanon'])
    else:
        outdict['filename'] = "%s-%s-%s" % (directorscanon, titlecanon, year)
    outdict['filename'] = ''.join(nchar for nchar in outdict['filename'] if nchar in string.printable)
    badchars='''\/:*?"<>'|~'''
    for nchar in badchars:
        outdict['filename'] = outdict['filename'].replace(nchar,'_')

    return(outdict)

def pickone(matches, title, year, interactive, verbose):
    acceptablekinds = ['movie', 'episode', 'video movie', 'tv miniseries']
    if not matches:
        return(None)
    else:
        candidates = []
        for entry in matches:
            try:
                entrykind = entry.get('kind')
                entrytitle = entry.get('smart long imdb canonical title')
                entryyear = entry.get('year')
            except:
                pass
            if entrykind in acceptablekinds:
                if entrytitle == title and entryyear == int(year):
                    if verbose:  ERR("Found an exact match for %s" % (entrytitle))
                    candidates.insert(0, entry)
                elif title in entrytitle and abs(entryyear - int(year)) <= 1:
                    if verbose:  ERR("Found a close match with %s" % (entrytitle))
                    candidates.insert(1, entry)
                else:
                    if verbose:  ERR("Found a poor match with %s" % (entrytitle))
                    candidates.append(entry)
            else:
                if verbose:  ERR("Skipping %s named %s" % (entrykind, entrytitle))
        if candidates:
            if interactive and len(candidates)>0:
                ERR("Pick the number that matches \"%s\" from \"%s\"" % (title, year))
                for index in range(len(candidates)):
                    entrykind = candidates[index].get('kind')
                    title = candidates[index].get('smart long imdb canonical title')
                    year = candidates[index].get('year')
                    ERR("%d :  %s :  %s %d" % (index, entrykind, title, year))
                choice = input('?:  ')
                try:
                    choice = int(choice)
                except:
                    ERR("Going with the first choice, then.")
                    choice = 0
                if choice >= len(candidates):
                    choice = None
                chosen = candidates.pop(choice)
                ERR("Picked %s from %s" % (chosen.get('smart long imdb canonical title'), chosen.get('year')))
                return(chosen)
            else:
                return(candidates[0])
        else:
            return(None)


def matchingnames(cursor, searchstring):
    '''Search IMDB for a particular movie.'''
    return(cursor.search_movie(str(searchstring)))





"""Console script for mkvimdb."""
import sys
import os
from optparse import OptionParser
import mkvimdb.mkvimdb as mi


def main(argv=None):
    """Console script for mkvimdb."""
    program_name = os.path.basename(sys.argv[0])
    program_version = "all"
    program_build_date = "%s" % 2020

    program_version_string = '%%prog %s (%s)' % (program_version,
                                                 program_build_date)
    program_longdesc = '''MKV tags from IMDB info, interactively or not.'''

    program_license = '''Copyright 2020 Dimitry Dukhovny'''

    if argv is None:
        argv = sys.argv[1:]
    try:
        # setup option parser
        parser = OptionParser(version=program_version_string,
                              epilog=program_longdesc,
                              description=program_license)
        parser.add_option("-v", "--verbose", dest="verbose",
                          action="store_true",
                          help="set verbosity level \
                          [default: %default]")
        parser.add_option("-I", "--interactive", dest="interactive",
                          action="store_true",
                          help="Select best match interactively instead of guessing. \
                          [default: %default]")
        parser.add_option("-s", "--search", dest="searchtitle",
                          action="store", type="string",
                          help="Set the search string. \
                          [default: %default]")
        parser.add_option("-y", "--year", dest="searchyear",
                          action="store", type="string",
                          help="Set the likely release year. \
                          [default: %default]")
        parser.add_option("-o", "--outputdir", dest="output_dir",
                          action="store", type="string",
                          help="Set the output directory, if any. \
                          [default: %default]")
        parser.add_option("-g", "--genre", dest="extragenre",
                          action="store", type="string",
                          help="Add comma-separated genre list. \
                          [default: %default]")
        parser.add_option("-G", "--replace-genre", dest="replacegenre",
                          action="store", type="string",
                          help="Replace discovered genre. \
                          [default: %default]")
        parser.add_option("-T", "--replace-title", dest="replacetitle",
                          action="store", type="string",
                          help="Replace discovered title. \
                          [default: %default]")

        # set defaults
        parser.set_defaults(verbose=False)
        parser.set_defaults(interactive=False)
        parser.set_defaults(searchtitle='Pulp Fiction')
        parser.set_defaults(searchyear='1994')
        parser.set_defaults(output_dir='')
        parser.set_defaults(extragenre='')
        parser.set_defaults(replacegenre='')
        parser.set_defaults(replacetitle='')

        # process options
        (opts, args) = parser.parse_args(argv)

        if opts.extragenre:
            opts.extragenre = opts.extraganre.replace(",", ", ")
        if opts.replacegenre:
            opts.replacegenre = opts.replaceganre.replace(",", ", ")

    except:
        return(2)

    m = mi.moviecursor(
        title=opts.searchtitle,
        year=opts.searchyear,
        output_dir=opts.output_dir,
        extragenre=opts.extragenre,
        replacegenre=opts.replacegenre,
        replacetitle=opts.replacetitle,
        interactive=opts.interactive,
        verbose=opts.verbose)

if __name__ == "__main__":
    sys.exit(main(sys.argv))  # pragma: no cover
